{% extends "base.html" %}

{% block title %}Mashpit{% endblock %}
{% block css %}.table thead th {
  text-align: left;
}
</style>{% endblock %}
{% block content %}
    <div class="container">
      <h1>
        <strong>Mashpit</strong>
        <br />
        <small class="text-muted fw-light">
          <span style="font-size: 0.5em;"> A sketch-based survilliance database</span>
        </small>
      </h1>
      <br />
      <div class="row">
        <div class="col-md-10 mx-auto">
          
        </div>
      </div>
    </div>
{% endblock %} 
